@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2019 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # Copyright (c) 2019 Jordi Romera <jordiromera@users.sourceforge.net>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@ from gi.repository.Gio import FILE_ATTRIBUTE_STANDARD_NAME, \
 from gettext import gettext as _
 from time import time
 import json
-from multiprocessing import cpu_count
 
 from lollypop.inotify import Inotify
 from lollypop.define import App, ScanType, Type, StorageType, ScanUpdate
@@ -32,7 +31,7 @@ from lollypop.tagreader import TagReader, Discoverer
 from lollypop.logger import Logger
 from lollypop.database_history import History
 from lollypop.utils import is_audio, is_pls, get_mtime, profile, create_dir
-from lollypop.utils import split_list, emit_signal
+from lollypop.utils import emit_signal
 
 
 SCAN_QUERY_INFO = "{},{},{},{},{},{}".format(
@@ -66,7 +65,6 @@ class CollectionScanner(GObject.GObject, TagReader):
         self.__history = History()
         self.__progress_total = 1
         self.__progress_count = 0
-        self.__new_tracks = []
         self.__new_non_album_artists = []
         self.__disable_compilations = True
         if App().settings.get_value("auto-update"):
@@ -227,7 +225,9 @@ class CollectionScanner(GObject.GObject, TagReader):
         # are playing with a compilation or an album
         self.__new_non_album_artists += list(
             set(added_artist_ids) - set(added_album_artist_ids))
-        if storage_type & StorageType.COLLECTION:
+        if storage_type & (StorageType.COLLECTION |
+                           StorageType.SPOTIFY_NEW_RELEASES |
+                           StorageType.SPOTIFY_SIMILARS):
             for artist_id in added_album_artist_ids:
                 if artist_id in self.__new_non_album_artists:
                     self.__new_non_album_artists.remove(artist_id)
@@ -435,17 +435,6 @@ class CollectionScanner(GObject.GObject, TagReader):
                 info = f.query_info(SCAN_QUERY_INFO,
                                     Gio.FileQueryInfoFlags.NONE,
                                     None)
-                # Ignore hidden files
-                if info.get_is_hidden():
-                    continue
-                # Ignore internal symlinks, we do not want to add tracks
-                # multiple times
-                if info.get_is_symlink():
-                    target = info.get_symlink_target()
-                    f = Gio.File.new_for_path(target)
-                    if self.__is_symlink_internal(f.get_uri(), uris):
-                        continue
-                # Add directory content to walked uris
                 if info.get_file_type() == Gio.FileType.DIRECTORY:
                     dirs.append(uri)
                     infos = f.enumerate_children(SCAN_QUERY_INFO,
@@ -453,7 +442,14 @@ class CollectionScanner(GObject.GObject, TagReader):
                                                  None)
                     for info in infos:
                         f = infos.get_child(info)
-                        walk_uris.append(f.get_uri())
+                        child_uri = f.get_uri()
+                        if info.get_is_hidden():
+                            continue
+                        # User do not want internal symlinks
+                        elif info.get_is_symlink() and\
+                                App().settings.get_value("ignore-symlinks"):
+                            continue
+                        walk_uris.append(child_uri)
                     infos.close(None)
                 # Only happens if files passed as args
                 else:
@@ -497,28 +493,17 @@ class CollectionScanner(GObject.GObject, TagReader):
         db_mtimes = App().tracks.get_mtimes()
         self.__progress_total = len(files)
         self.__progress_count = 0
-        split_files = split_list(files, max(1, cpu_count() // 2))
-        self.__new_tracks = []
-        threads = []
-        for files in split_files:
-            thread = App().task_helper.run(self.__scan_files, files, db_mtimes,
-                                           scan_type)
-            threads.append(thread)
-
-        # Wait for scan to finish
-        for thread in threads:
-            thread.join()
-
+        new_tracks = self.__scan_files(files, db_mtimes, scan_type)
         self.__remove_old_tracks(db_uris, scan_type)
 
         SqlCursor.remove(App().db)
 
         if scan_type != ScanType.EXTERNAL:
             self.__add_monitor(dirs)
-            GLib.idle_add(self.__finish, self.__new_tracks)
+            GLib.idle_add(self.__finish, new_tracks)
 
         if scan_type == ScanType.EXTERNAL:
-            App().player.play_uris(self.__new_tracks)
+            App().player.play_uris(new_tracks)
 
     def __scan_to_handle(self, uri):
         """
@@ -550,6 +535,7 @@ class CollectionScanner(GObject.GObject, TagReader):
             @return new track uris as [str]
             @thread safe
         """
+        new_tracks = []
         SqlCursor.add(App().db)
         discoverer = Discoverer()
         try:
@@ -573,7 +559,7 @@ class CollectionScanner(GObject.GObject, TagReader):
                             mtime = int(time())
                         Logger.debug("Adding file: %s" % uri)
                         self.__add2db(discoverer, uri, mtime, storage_type)
-                        self.__new_tracks.append(uri)
+                        new_tracks.append(uri)
                     if db_mtime != 0:
                         del db_mtimes[uri]
                 except Exception as e:
@@ -589,6 +575,7 @@ class CollectionScanner(GObject.GObject, TagReader):
                 emit_signal(self, "artist-updated",
                             artist_id, ScanUpdate.ADDED)
         SqlCursor.remove(App().db)
+        return new_tracks
 
     def __remove_old_tracks(self, uris, scan_type):
         """
@@ -711,14 +698,3 @@ class CollectionScanner(GObject.GObject, TagReader):
                         tracknumber, discnumber, discname, year, timestamp,
                         track_mtime, track_pop, track_rate, track_loved,
                         track_ltime, mb_track_id, bpm, storage_type)
-
-    def __is_symlink_internal(self, uri, uris):
-        """
-            True if symlink as uri is included in uris
-            @param uri as str
-            @param uris as [str]
-        """
-        for base_uri in uris:
-            if uri.startswith(base_uri):
-                return True
-        return False
