@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2019 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -16,10 +16,10 @@ import json
 from base64 import b64encode
 
 from lollypop.define import App, GOOGLE_API_ID, Type, AUDIODB_CLIENT_ID
-from lollypop.define import SPOTIFY_CLIENT_ID, SPOTIFY_SECRET
-from lollypop.utils import get_network_available, noaccents
+from lollypop.define import SPOTIFY_CLIENT_ID, SPOTIFY_SECRET, FANARTTV_ID
+from lollypop.utils import get_network_available, noaccents, emit_signal
 from lollypop.logger import Logger
-from lollypop.objects import Album
+from lollypop.objects_album import Album
 from lollypop.downloader import Downloader
 from lollypop.helper_task import TaskHelper
 
@@ -35,9 +35,9 @@ class ArtDownloader(Downloader):
         """
         Downloader.__init__(self)
         self.__albums_queue = []
-        self.__albums_history = []
+        self.__artists_queue = []
         self.__in_albums_download = False
-        self.__cache_artists_running = False
+        self.__in_artists_download = False
 
     def search_album_artworks(self, artist, album, cancellable):
         """
@@ -52,10 +52,10 @@ class ArtDownloader(Downloader):
             if helper is None:
                 continue
             method = getattr(self, helper)
-            uri = method(artist, album, cancellable)
-            if uri is not None:
+            uris = method(artist, album, cancellable)
+            for uri in uris:
                 results.append((uri, api))
-        GLib.idle_add(self.emit, "uri-artwork-found", results)
+        emit_signal(self, "uri-artwork-found", results)
 
     def search_artist_artwork(self, artist, cancellable):
         """
@@ -69,31 +69,32 @@ class ArtDownloader(Downloader):
             if helper is None:
                 continue
             method = getattr(self, helper)
-            uri = method(artist, cancellable)
-            if uri is not None:
+            uris = method(artist, cancellable)
+            for uri in uris:
                 results.append((uri, api))
-        GLib.idle_add(self.emit, "uri-artwork-found", results)
+        emit_signal(self, "uri-artwork-found", results)
 
     def cache_album_artwork(self, album_id):
         """
             Download album artwork
             @param album_id as int
         """
-        if album_id in self.__albums_history or\
-                not get_network_available("DATA"):
+        if not get_network_available("DATA"):
             return
         self.__albums_queue.append(album_id)
         if not self.__in_albums_download:
-            App().task_helper.run(self.__cache_albums_art)
+            App().task_helper.run(self.__cache_albums_artwork)
 
-    def cache_artists_artwork(self):
+    def cache_artist_artwork(self, artist):
         """
-            Cache info for all artists
+            Cache artist artwork
+            @param artist as str
         """
-        if self.__cache_artists_running or not get_network_available("DATA"):
+        if not get_network_available("DATA"):
             return
-        self.__cache_artists_running = True
-        App().task_helper.run(self.__cache_artists_artwork)
+        self.__artists_queue.append(artist)
+        if not self.__in_artists_download:
+            App().task_helper.run(self.__cache_artists_artwork)
 
     def search_artwork_from_google(self, search, cancellable):
         """
@@ -102,7 +103,7 @@ class ArtDownloader(Downloader):
             @param cancellable as Gio.Cancellable
         """
         if not get_network_available("GOOGLE"):
-            GLib.idle_add(self.emit, "uri-artwork-found", None)
+            emit_signal(self, "uri-artwork-found", None)
             return
         key = App().settings.get_value("cs-api-key").get_string() or\
             App().settings.get_default_value("cs-api-key").get_string()
@@ -122,7 +123,7 @@ class ArtDownloader(Downloader):
             @param cancellable as Gio.Cancellable
         """
         if not get_network_available("STARTPAGE"):
-            GLib.idle_add(self.emit, "uri-artwork-found", None)
+            emit_signal(self, "uri-artwork-found", None)
             return
         uri = "https://www.startpage.com/do/search?flimgsize=isz%3Al"
         uri += "&image-size-select=&flimgexwidth=&flimgexheight=&abp=-1"
@@ -131,52 +132,105 @@ class ArtDownloader(Downloader):
                                            cancellable,
                                            self.__on_load_startpage_content)
 
-    def reset_history(self):
-        """
-            Reset download history
-        """
-        self.__albums_history = []
-
 #######################
 # PROTECTED           #
 #######################
+    def _get_audiodb_artist_artwork_uri(self, artist, cancellable=None):
+        """
+            Get artist artwork using AutdioDB
+            @param artist as str
+            @param cancellable as Gio.Cancellable
+            @return uri as str
+            @thread safe
+        """
+        if not get_network_available("AUDIODB"):
+            return []
+        try:
+            artist = GLib.uri_escape_string(artist, None, True)
+            uri = "https://theaudiodb.com/api/v1/json/"
+            uri += "%s/search.php?s=%s" % (AUDIODB_CLIENT_ID, artist)
+            (status, data) = App().task_helper.load_uri_content_sync(
+                uri, cancellable)
+            if status:
+                decode = json.loads(data.decode("utf-8"))
+                uri = None
+                for item in decode["artists"]:
+                    for key in ["strArtistFanart", "strArtistThumb"]:
+                        uri = item[key]
+                        if uri is not None:
+                            return [uri]
+        except Exception as e:
+            Logger.warning(
+                "ArtDownloader::_get_audiodb_artist_artwork_uri: %s : %s"
+                % (e, artist))
+        return []
+
     def _get_deezer_artist_artwork_uri(self, artist, cancellable=None):
         """
-            Return deezer artist information
+            Get artist artwork using Deezer
             @param artist as str
             @param cancellable as Gio.Cancellable
             @return uri as str
             @tread safe
         """
         if not get_network_available("DEEZER"):
-            return None
+            return []
         try:
             artist_formated = GLib.uri_escape_string(
                 artist, None, True).replace(" ", "+")
             uri = "https://api.deezer.com/search/artist/?" +\
-                  "q=%s&output=json&index=0&limit=1&" % artist_formated
+                  "q=%s&output=json&index=0&limit=1" % artist_formated
             (status, data) = App().task_helper.load_uri_content_sync(
                 uri, cancellable)
             if status:
                 uri = None
                 decode = json.loads(data.decode("utf-8"))
                 uri = decode["data"][0]["picture_xl"]
-                return uri
+                return [uri]
         except Exception as e:
-            Logger.debug("ArtDownloader::_get_deezer_artist_artwork_uri(): %s"
-                         % e)
-        return None
+            Logger.warning(
+                "ArtDownloader::_get_deezer_artist_artwork_uri(): %s : %s"
+                % (e, artist))
+        return []
+
+    def _get_fanarttv_artist_artwork_uri(self, artist, cancellable=None):
+        """
+            Get artist artwork using FanartTV
+            @param artist as str
+            @param cancellable as Gio.Cancellable
+            @return uri as str
+            @thread safe
+        """
+        if not get_network_available("FANARTTV"):
+            return []
+        uris = []
+        try:
+            mbid = self.__get_musicbrainz_mbid("artist", artist, cancellable)
+            if mbid is None:
+                return []
+            uri = "http://webservice.fanart.tv/v3/music/%s?api_key=%s"
+            (status, data) = App().task_helper.load_uri_content_sync(
+                uri % (mbid, FANARTTV_ID), cancellable)
+            if status:
+                decode = json.loads(data.decode("utf-8"))
+                for item in decode["artistbackground"]:
+                    uris.append(item["url"])
+        except Exception as e:
+            Logger.warning(
+                "ArtDownloader::_get_fanarttv_artist_artwork_uri: %s : %s"
+                % (e, artist))
+        return uris
 
     def _get_spotify_artist_artwork_uri(self, artist, cancellable=None):
         """
-            Return spotify artist information
+            Get artist artwork using Spotify
             @param artist as str
             @param cancellable as Gio.Cancellable
             @return uri as str
             @tread safe
         """
         if not get_network_available("SPOTIFY"):
-            return None
+            return []
         try:
             artist_formated = GLib.uri_escape_string(
                 artist, None, True).replace(" ", "+")
@@ -193,15 +247,16 @@ class ArtDownloader(Downloader):
                     if noaccents(item["name"].lower()) ==\
                             noaccents(artist.lower()):
                         uri = item["images"][0]["url"]
-                        return uri
+                        return [uri]
         except Exception as e:
-            Logger.debug("ArtDownloader::_get_spotify_artist_artwork_uri(): %s"
-                         % e)
-        return None
+            Logger.warning(
+                "ArtDownloader::_get_spotify_artist_artwork_uri(): %s : %s"
+                % (e, artist))
+        return []
 
     def _get_deezer_album_artwork_uri(self, artist, album, cancellable=None):
         """
-            Get album artwork uri from deezer
+            Get album artwork using Deezer
             @param artist as str
             @param album as str
             @param cancellable as Gio.Cancellable
@@ -209,7 +264,7 @@ class ArtDownloader(Downloader):
             @tread safe
         """
         if not get_network_available("DEEZER"):
-            return None
+            return []
         try:
             album_formated = GLib.uri_escape_string(album, None, True)
             uri = "https://api.deezer.com/search/album/?" +\
@@ -223,15 +278,44 @@ class ArtDownloader(Downloader):
                     if noaccents(item["artist"]["name"].lower()) ==\
                             noaccents(artist.lower()):
                         uri = item["cover_xl"]
-                        return uri
+                        return [uri]
         except Exception as e:
-            Logger.error("ArtDownloader::__get_deezer_album_artwork_uri: %s"
-                         % e)
-        return None
+            Logger.warning("ArtDownloader::__get_deezer_album_artwork_uri: %s"
+                           % e)
+        return []
+
+    def _get_fanarttv_album_artwork_uri(self, artist, album, cancellable=None):
+        """
+            Get album artwork using FanartTV
+            @param artist as str
+            @param album as str
+            @param cancellable as Gio.Cancellable
+            @return uri as str
+            @thread safe
+        """
+        if not get_network_available("FANARTTV"):
+            return []
+        uris = []
+        try:
+            search = "%s %s" % (artist, album)
+            mbid = self.__get_musicbrainz_mbid("album", search, cancellable)
+            if mbid is None:
+                return []
+            uri = "http://webservice.fanart.tv/v3/music/albums/%s?api_key=%s"
+            (status, data) = App().task_helper.load_uri_content_sync(
+                uri % (mbid, FANARTTV_ID), cancellable)
+            if status:
+                decode = json.loads(data.decode("utf-8"))
+                for cover in decode["albums"][mbid]["albumcover"]:
+                    uris.append(cover["url"])
+        except Exception as e:
+            Logger.warning(
+                "ArtDownloader::_get_fanarttv_album_artwork_uri: %s" % e)
+        return uris
 
     def _get_spotify_album_artwork_uri(self, artist, album, cancellable=None):
         """
-            Get album artwork uri from spotify
+            Get album artwork using Spotify
             @param artist as str
             @param album as str
             @param cancellable as Gio.Cancellable
@@ -239,7 +323,7 @@ class ArtDownloader(Downloader):
             @tread safe
         """
         if not get_network_available("SPOTIFY"):
-            return None
+            return []
         artists_spotify_ids = []
         try:
             token = self.__get_spotify_token(cancellable)
@@ -266,14 +350,14 @@ class ArtDownloader(Downloader):
                     for item in decode["items"]:
                         if noaccents(item["name"].lower()) ==\
                                 noaccents(album.lower()):
-                            return item["images"][0]["url"]
+                            return [item["images"][0]["url"]]
         except Exception as e:
-            Logger.error("ArtDownloader::_get_album_art_spotify_uri: %s" % e)
-        return None
+            Logger.warning("ArtDownloader::_get_album_art_spotify_uri: %s" % e)
+        return []
 
     def _get_itunes_album_artwork_uri(self, artist, album, cancellable=None):
         """
-            Get album artwork uri from itunes
+            Get album artwork using Itunes
             @param artist as str
             @param album as str
             @param cancellable as Gio.Cancellable
@@ -281,7 +365,7 @@ class ArtDownloader(Downloader):
             @tread safe
         """
         if not get_network_available("ITUNES"):
-            return None
+            return []
         try:
             album_formated = GLib.uri_escape_string(
                 album, None, True).replace(" ", "+")
@@ -296,15 +380,15 @@ class ArtDownloader(Downloader):
                             noaccents(artist.lower()):
                         uri = item["artworkUrl60"].replace("60x60",
                                                            "1024x1024")
-                        return uri
+                        return [uri]
         except Exception as e:
-            Logger.error("ArtDownloader::_get_album_art_itunes_uri: %s"
-                         % e)
-        return None
+            Logger.warning("ArtDownloader::_get_album_art_itunes_uri: %s"
+                           % e)
+        return []
 
     def _get_audiodb_album_artwork_uri(self, artist, album, cancellable=None):
         """
-            Get album artwork from audiodb
+            Get album artwork using AudioDB
             @param artist as str
             @param album as str
             @param cancellable as Gio.Cancellable
@@ -312,7 +396,7 @@ class ArtDownloader(Downloader):
             @thread safe
         """
         if not get_network_available("AUDIODB"):
-            return None
+            return []
         try:
             album = GLib.uri_escape_string(album, None, True)
             artist = GLib.uri_escape_string(artist, None, True)
@@ -327,65 +411,76 @@ class ArtDownloader(Downloader):
                 if decode["album"]:
                     for item in decode["album"]:
                         uri = item["strAlbumThumb"]
-                        return uri
+                        return [uri]
         except Exception as e:
-            Logger.error("ArtDownloader::_get_audiodb_album_artwork_uri: %s"
-                         % e)
-        return None
-
-    def _get_audiodb_artist_artwork_uri(self, artist, cancellable=None):
-        """
-            Get artist artwork uri from audiodb
-            @param artist as str
-            @param cancellable as Gio.Cancellable
-            @return uri as str
-            @thread safe
-        """
-        if not get_network_available("AUDIODB"):
-            return None
-        try:
-            artist = GLib.uri_escape_string(artist, None, True)
-            uri = "https://theaudiodb.com/api/v1/json/"
-            uri += "%s/search.php?s=%s" % (AUDIODB_CLIENT_ID, artist)
-            (status, data) = App().task_helper.load_uri_content_sync(
-                uri, cancellable)
-            if status:
-                decode = json.loads(data.decode("utf-8"))
-                uri = None
-                for item in decode["artists"]:
-                    for key in ["strArtistFanart", "strArtistThumb"]:
-                        uri = item[key]
-                        if uri is not None:
-                            return uri
-        except Exception as e:
-            Logger.error("ArtDownloader::_get_audiodb_artist_artwork_uri: %s"
-                         % e)
-        return None
+            Logger.warning("ArtDownloader::_get_audiodb_album_artwork_uri: %s"
+                           % e)
+        return []
 
     def _get_lastfm_album_artwork_uri(self, artist, album, cancellable=None):
         """
-            Get album artwork uri from lastfm
+            Get album artwork using Last.FM
             @param artist as str
             @param album as str
             @param cancellable as Gio.Cancellable
-            @return uri as str
+            @return [str]
             @tread safe
         """
         if not get_network_available("LASTFM"):
-            return None
+            return []
         if App().lastfm is not None:
             try:
                 last_album = App().lastfm.get_album(artist, album)
                 uri = last_album.get_cover_image(4)
-                return uri
+                return [uri]
             except Exception as e:
-                Logger.error("ArtDownloader::_get_album_art_lastfm_uri: %s"
-                             % e)
-        return None
+                Logger.warning("ArtDownloader::_get_album_art_lastfm_uri: %s"
+                               % e)
+        return []
 
 #######################
 # PRIVATE             #
 #######################
+    def __get_musicbrainz_mbid(self, mbid_type, string, cancellable):
+        """
+            Get musicbrainz mbid for type and string
+            @param mbid_type as str ("artist" or "album")
+            @param string as str
+            @param cancellable as Gio.Cancellable
+        """
+        try:
+            if mbid_type == "artist":
+                uri = "http://musicbrainz.org/ws/2/artist/" +\
+                      "?query=%s&fmt=json"
+            else:
+                uri = "http://musicbrainz.org/ws/2/release-group/" +\
+                      "?query=%s&fmt=json"
+            string = GLib.uri_escape_string(string, None, True)
+            helper = TaskHelper()
+            helper.add_header("User-Agent",
+                              "org.gnome.Lollypop/%s" % App().version)
+            (status, data) = helper.load_uri_content_sync(uri % string,
+                                                          cancellable)
+            if status:
+                decode = json.loads(data.decode("utf-8"))
+                if mbid_type == "artist":
+                    for item in decode["artists"]:
+                        return item["id"]
+                else:
+                    mbid = None
+                    # Get album id or EP id if missing
+                    for item in decode["release-groups"]:
+                        if item["primary-type"] == "Album":
+                            mbid = item["id"]
+                            break
+                        elif item["primary-type"] == "EP" and mbid is None:
+                            mbid = item["id"]
+                return mbid
+        except Exception as e:
+            Logger.warning("ArtDownloader::__get_musicbrainz_mbid: %s"
+                           % e)
+        return None
+
     def __get_spotify_token(self, cancellable):
         """
             Get a new auth token
@@ -415,30 +510,35 @@ class ArtDownloader(Downloader):
         """
             Cache artwork for all artists
         """
-        # Then cache for lastfm/spotify/deezer/...
-        for (artist_id, artist, sort) in App().artists.get([]):
-            if App().art.artist_artwork_exists(artist)[0]:
-                continue
-            for (api, helper, a_helper, b_helper) in self._WEBSERVICES:
-                if helper is None:
-                    continue
-                try:
+        self.__in_artists_download = True
+        try:
+            while self.__artists_queue:
+                artist = self.__artists_queue.pop()
+                found = False
+                for (api, helper, a_helper, b_helper) in self._WEBSERVICES:
+                    if helper is None:
+                        continue
                     method = getattr(self, helper)
-                    uri = method(artist)
-                    if uri is not None:
+                    result = method(artist)
+                    for uri in result:
                         (status,
-                         data) = App().task_helper.load_uri_content_sync(uri,
-                                                                         None)
+                         data) = App().task_helper.load_uri_content_sync(
+                            uri, None)
                         if status:
+                            found = True
                             App().art.add_artist_artwork(artist, data)
                             break
-                except Exception as e:
-                    Logger.error(
-                        "ArtDownloader::__cache_artists_artwork(): %s" % e)
+                    # Found, do not search in another helper
+                    if found:
+                        break
+                # Not found, save empty artwork
+                if not found:
                     App().art.add_artist_artwork(artist, None)
-        self.__cache_artists_running = False
+        except Exception as e:
+            Logger.error("ArtDownloader::__cache_artists_artwork(): %s" % e)
+        self.__in_artists_download = False
 
-    def __cache_albums_art(self):
+    def __cache_albums_artwork(self):
         """
             Cache albums artwork (from queue)
             @thread safe
@@ -455,22 +555,29 @@ class ArtDownloader(Downloader):
                     artist = ""
                 else:
                     artist = ", ".join(App().albums.get_artists(album_id))
+                found = False
                 for (api, a_helper, helper, b_helper) in self._WEBSERVICES:
                     if helper is None:
                         continue
                     method = getattr(self, helper)
-                    uri = method(artist, album)
-                    if uri is not None:
+                    result = method(artist, album)
+                    for uri in result:
                         (status,
                          data) = App().task_helper.load_uri_content_sync(uri,
                                                                          None)
                         if status:
-                            self.__albums_history.append(album_id)
+                            found = True
                             App().art.save_album_artwork(data, Album(album_id))
                             break
+                    # Found, do not search in another helper
+                    if found:
+                        break
+                # Not found, save empty artwork
+                # FIXME move album as first arg
+                if not found:
+                    App().art.save_album_artwork(None, Album(album_id))
         except Exception as e:
-            Logger.error("ArtDownloader::__cache_albums_art: %s" % e)
-        self.__albums_history.append(album_id)
+            Logger.error("ArtDownloader::__cache_albums_artwork: %s" % e)
         self.__in_albums_download = False
 
     def __on_load_google_content(self, uri, loaded, content):
@@ -482,16 +589,16 @@ class ArtDownloader(Downloader):
         """
         try:
             if not loaded:
-                self.emit("uri-artwork-found", [])
+                emit_signal(self, "uri-artwork-found", [])
                 return
             decode = json.loads(content.decode("utf-8"))
             results = []
             for item in decode["items"]:
                 if item["link"] is not None:
                     results.append((item["link"], "Google"))
-            self.emit("uri-artwork-found", results)
+            emit_signal(self, "uri-artwork-found", results)
         except Exception as e:
-            self.emit("uri-artwork-found", [])
+            emit_signal(self, "uri-artwork-found", [])
             Logger.error("ArtDownloader::__on_load_google_content(): %s: %s"
                          % (e, content))
 
@@ -502,22 +609,31 @@ class ArtDownloader(Downloader):
             @param loaded as bool
             @param content as bytes
         """
+        import re
+
+        def search_in_data(lines, found_uris=[]):
+            if lines:
+                line = lines.pop(0)
+                # Do not call findall if nothing to find
+                if line.find("oiu=") != -1:
+                    res = re.findall(r'.*oiu=([^&]*).*', line)
+                    for data in res:
+                        uri = GLib.uri_unescape_string(data, "")
+                        if uri in found_uris or uri is None:
+                            continue
+                        found_uris.append(uri)
+                GLib.idle_add(search_in_data, lines, found_uris)
+            else:
+                results = [(uri, "Startpage") for uri in found_uris]
+                emit_signal(self, "uri-artwork-found", results)
+
         try:
             if not loaded:
-                self.emit("uri-artwork-found", [])
+                emit_signal(self, "uri-artwork-found", [])
                 return
-            found_uris = []
-            import re
-            data = content.decode("utf-8")
-            res = re.findall(r'.*oiu=([^&]*).*', data)
-            for data in res:
-                uri = GLib.uri_unescape_string(data, "")
-                if uri in found_uris or uri is None:
-                    continue
-                found_uris.append(uri)
-            results = [(uri, "Startpage") for uri in found_uris]
-            self.emit("uri-artwork-found", results)
+            lines = content.decode("utf-8").splitlines()
+            search_in_data(lines)
         except Exception as e:
-            self.emit("uri-artwork-found", [])
+            emit_signal(self, "uri-artwork-found", [])
             Logger.error("ArtDownloader::__on_load_startpage_content(): %s"
                          % e)
