@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2019 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -75,8 +75,7 @@ class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
         self.__progress_widget.show()
         self.__progress_widget.set_property("halign", Gtk.Align.CENTER)
         self.__progress_widget.set_size_request(500, -1)
-        self.__buttons_widget = ButtonsPlayerWidget(["menu-button",
-                                                     "black-transparent"])
+        self.__buttons_widget = ButtonsPlayerWidget(["banner-button"])
         self.__buttons_widget.show()
         self.__buttons_widget.set_size_request(500, -1)
         self.__buttons_widget.set_property("valign", Gtk.Align.CENTER)
@@ -214,7 +213,7 @@ class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
 
     @property
     def adaptive_size(self):
-        return AdaptiveSize.SMALL
+        return AdaptiveSize.PHONE
 
     @property
     def container(self):
