@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2019 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -35,7 +35,6 @@ class Discoverer:
         """
             Init discover
         """
-        GstPbutils.pb_utils_init()
         self._discoverer = GstPbutils.Discoverer.new(10 * Gst.SECOND)
 
     def get_info(self, uri):
@@ -458,6 +457,9 @@ class TagReader:
                     - Windows Media Player is located in m.data[40] and
                       we are searching m.data[10:40] for
                       "Windows Media Player 9 Series"
+                    - MusicBee is located in m.data[19] and
+                      we are searching m.data[10:40] for
+                      "MusicBee"
                 """
                 if m.data[10:40].find(b"Windows Media Player 9 Series") >= 0:
                     location = 40
@@ -474,6 +476,11 @@ class TagReader:
                     Logger.debug(
                         "Rating type: MediaMonkey Location: %s Rating: %s",
                         location, m.data[location])
+                elif m.data[10:40].find(b"MusicBee") >= 0:
+                    location = 19
+                    Logger.debug(
+                        "Rating type: MusicBee Location: %s Rating: %s",
+                        location, m.data[location])
                 else:
                     location = 11
                     Logger.debug(
